% GatorLaunch.m
% Launch Dispersion Study with STK/Astrogator
% Analytical Graphics, Inc. 1-800-220-4785
% Space Exploration Engineering, Inc. (360)378-7168
% Original Perl Script Written by John Carrico(AGI) and Mike Shepherd (AGI), March, 2000
% Matlab Conversion written by Josh Lane (AGI)
% Additional Matlab cleanup and debugging by Mike Loucks (SEE) Aug, 2001.
% Histograms, Delta-V added by John Carrico (AGI) Nov, 2001

% This script requires the "LaunchErrors" scenario from the Astrogator Training Material

% This script also requires the 'StripGatorValue.m' Matlab file.

disp('Matlab STK/Astrogator Launch Dispersion Study');
disp('Space Exploration Engineering, Inc.');
disp('Analytical Graphics, Inc.');

% Clear any previous values from the workspace

clear;

% Enter Iteration Value

numPoints = input('Enter the number of trial points (10 to demo, 400 to 1000 for real): ');

%sma = zeros(numPoints,1);
%ecc = zeros(numPoints,1);
%inc = zeros(numPoints,1);

sigAlt = 5;
sigLat = 0.1;
sigLon = 0.1;
sigVel = 0.001;

sat = ['*/Satellite/Sat1'];

launchObj = ['MainSEQUENCE.SegmentList.WallopsLaunch'];
EvalObj   = ['MainSEQUENCE.SegmentList.Prop10Mins'];
mnvr1     = ['MainSEQUENCE.SegmentList.RaiseApogee.SegmentList.Mnvr1'];
mnvr2     = ['MainSEQUENCE.SegmentList.RaisePerigee.SegmentList.Mnvr2'];

start_time = ['15 Dec 2001 00:00:00.00'];
stop_time  = ['25 Jan 2002 00:00:00.00'];

%--- Connect to STK and initialize

conid = stkopen('localhost:5001');
rtn = stkConnect(conid,'ConControl','/ VerboseOn AckOn');
disp('Connected to STK');

%--- Get initial control parameters

tempLatitude      = stkExec(conid,'ASTROGATOR_RM */Satellite/Sat1 GETVALUE MainSEQUENCE.SegmentList.WallopsLaunch.Burnout.Geodetic.Latitude');
tempLongitude     = stkExec(conid,'ASTROGATOR_RM */Satellite/Sat1 GETVALUE MainSEQUENCE.SegmentList.WallopsLaunch.Burnout.Geodetic.Longitude');
tempAltitude      = stkExec(conid,'ASTROGATOR_RM */Satellite/Sat1 GETVALUE MainSEQUENCE.SegmentList.WallopsLaunch.Burnout.Geodetic.Altitude');
tempFixedVelocity = stkExec(conid,'ASTROGATOR_RM */Satellite/Sat1 GETVALUE MainSEQUENCE.SegmentList.WallopsLaunch.Burnout.FixedVelocity');

initLatitude      = StripGatorValue(tempLatitude);
initLongitude     = StripGatorValue(tempLongitude);
initAltitude      = StripGatorValue(tempAltitude);
initFixedVelocity = StripGatorValue(tempFixedVelocity);

% Print out initial Values 
disp(' ');

printtemp = [num2str(initAltitude) ' km   ' num2str(initLatitude) ' deg   ' num2str(initLongitude) ' deg   ' num2str(initFixedVelocity) ' km/sec'];
disp('Initial Values');
disp(' Altitude       Latitude      Longitude     Velocity');
disp(printtemp);
disp(' ');

%--- Run Monte Carlo

printtemp = [ 'Running ' num2str(numPoints) ' Monte Carlo trials.'];
disp(printtemp);
disp('#   Altitude       Latitude      Longitude     Velocity');


for k = 1:numPoints

	% --- Set perturbed control parameters
	
    Altitude     = initAltitude + rand*(2.0*sigAlt) - sigAlt;
	Latitude     = initLatitude + rand*(2.0*sigLat) - sigLat;
	Longitude    = initLongitude + rand*(2.0*sigLon) - sigLon;
	FixedVelocity = initFixedVelocity + rand*(2.0*sigVel) - sigVel;
    
    printtemp = [num2str(k) '   ' num2str(Altitude) ' km   ' num2str(Latitude) ' deg   ' num2str(Longitude) ' deg   ' num2str(FixedVelocity) ' km/sec'];
    disp(printtemp);
	   
   varib = [launchObj '.Burnout.Geodetic.Altitude ' num2str(Altitude) ' km'];   
   rtn = stkConnect(conid,'SETGATORVALUES',sat,varib);
   
   varib = [launchObj '.Burnout.Geodetic.Latitude ' num2str(Latitude) ' deg'];
   rtn = stkConnect(conid,'SETGATORVALUES',sat,varib);
   
   varib = [launchObj '.Burnout.Geodetic.Longitude ' num2str(Longitude) ' deg'];
   rtn = stkConnect(conid,'SETGATORVALUES',sat,varib);
   
   varib = [launchObj '.Burnout.FixedVelocity ' num2str(FixedVelocity) ' km/sec'];
   rtn = stkConnect(conid,'SETGATORVALUES',sat,varib);
	
	% --- Launch the satellite
    
   q = '"';
   times = [q start_time q  ' ' q stop_time q ];
   
   rtn = stkConnect(conid,'Propagate',sat,times);
	
	%--- Get Evaluation values
   
   varib = ['GETVALUE ' EvalObj '.FinalState.Keplerian.sma'];
   smatemp  = stkConnect(conid,'ASTROGATOR_RM',sat,varib);
   sma(k)   = StripGatorValue(smatemp);
   
   varib = ['GETVALUE ' EvalObj '.FinalState.Keplerian.ecc'];
   ecctemp = stkConnect(conid,'ASTROGATOR_RM',sat,varib);
   ecc(k)   = StripGatorValue(ecctemp);
   
   varib = ['GETVALUE ' EvalObj '.FinalState.Keplerian.inc'];
   inctemp  = stkConnect(conid,'ASTROGATOR_RM',sat,varib);
   inc(k)   = StripGatorValue(inctemp);
   
   varib = ['GETVALUE ' EvalObj '.FinalState.Keplerian.RAAN'];
   raantemp  = stkConnect(conid,'ASTROGATOR_RM',sat,varib);
   raan(k)   = StripGatorValue(raantemp);
   
   varib = ['GETVALUE ' EvalObj '.FinalState.Keplerian.w'];
   wtemp  = stkConnect(conid,'ASTROGATOR_RM',sat,varib);
   w(k)   = StripGatorValue(wtemp);
   
   varib = ['GETVALUE ' EvalObj '.FinalState.Keplerian.TA'];
   TAtemp  = stkConnect(conid,'ASTROGATOR_RM',sat,varib);
   TA(k)   = StripGatorValue(TAtemp);
   
   alat(k) = TA(k) + w(k);
   
   varib = ['GETVALUE ' mnvr1 '.FinalDeltaV'];
   DV1temp  = stkConnect(conid,'ASTROGATOR_RM',sat,varib);
   DV1   = StripGatorValue(DV1temp);
   
   varib = ['GETVALUE ' mnvr2 '.FinalDeltaV'];
   DV2temp  = stkConnect(conid,'ASTROGATOR_RM',sat,varib);
   DV2   = StripGatorValue(DV2temp);
   
   DVMag(k) = DV1 + DV2;
   
end	

%--- Reset control parameters to initial values

disp('Resetting initial values...');

varib = [launchObj '.Burnout.Geodetic.Altitude ' num2str(initAltitude) ' km'];
rtn = stkConnect(conid,'SETGATORVALUES',sat,varib);

varib = [launchObj '.Burnout.Geodetic.Latitude ' num2str(initLatitude) ' deg'];
rtn = stkConnect(conid,'SETGATORVALUES',sat,varib);

varib = [launchObj '.Burnout.Geodetic.Longitude '  num2str(initLongitude) ' deg'];
rtn = stkConnect(conid,'SETGATORVALUES',sat,varib);

varib = [launchObj '.Burnout.FixedVelocity ' num2str(initFixedVelocity) ' km/sec'];
rtn = stkConnect(conid,'SETGATORVALUES',sat,varib);

times = [q start_time q  ' ' q stop_time q ];
rtn = stkConnect(conid,'Propagate',sat,times);

% Get Nominal Values of orbital elements

varib = ['GETVALUE ' EvalObj '.FinalState.Keplerian.sma'];
smatemp  = stkConnect(conid,'ASTROGATOR_RM',sat,varib);
nom_sma   = StripGatorValue(smatemp);
   
varib = ['GETVALUE ' EvalObj '.FinalState.Keplerian.ecc'];
ecctemp = stkConnect(conid,'ASTROGATOR_RM',sat,varib);
nom_ecc   = StripGatorValue(ecctemp);
   
varib = ['GETVALUE ' EvalObj '.FinalState.Keplerian.inc'];
inctemp  = stkConnect(conid,'ASTROGATOR_RM',sat,varib);
nom_inc  = StripGatorValue(inctemp);
   
varib = ['GETVALUE ' EvalObj '.FinalState.Keplerian.RAAN'];
ranntemp  = stkConnect(conid,'ASTROGATOR_RM',sat,varib);
nom_raan   = StripGatorValue(raantemp);
   
varib = ['GETVALUE ' EvalObj '.FinalState.Keplerian.w'];
wtemp  = stkConnect(conid,'ASTROGATOR_RM',sat,varib);
nom_w   = StripGatorValue(wtemp);
   
varib = ['GETVALUE ' EvalObj '.FinalState.Keplerian.TA'];
TAtemp  = stkConnect(conid,'ASTROGATOR_RM',sat,varib);
nom_TA   = StripGatorValue(TAtemp);
   
nom_alat = nom_w + nom_TA;

if nom_alat > 360.0 
	nom_alat = nom_alat - 360.0;
end

varib = ['GETVALUE ' mnvr1 '.FinalDeltaV'];
DV1temp  = stkConnect(conid,'ASTROGATOR_RM',sat,varib);
DV1   = StripGatorValue(DV1temp);
   
varib = ['GETVALUE ' mnvr2 '.FinalDeltaV'];
DV2temp  = stkConnect(conid,'ASTROGATOR_RM',sat,varib);
DV2   = StripGatorValue(DV2temp);

nom_DVMag = DV1 + DV2;

%------------------------------
%--- Calculate statistics

%--- Calculate average

tmp_sma   = 0;
tmp_ecc   = 0;
tmp_inc   = 0;
tmp_raan  = 0;
tmp_w     = 0;
tmp_TA    = 0;
tmp_alat  = 0;
tmp_DVMag = 0;

for i= 1:numPoints
	tmp_sma   = sma(i)   + tmp_sma;
	tmp_ecc   = ecc(i)   + tmp_ecc;
	tmp_inc   = inc(i)   + tmp_inc;
    tmp_raan  = raan(i)  + tmp_raan;
    tmp_w     = w(i)     + tmp_w;
    tmp_TA    = TA(i)    + tmp_TA;
    tmp_alat  = alat(i)  + tmp_alat;
    tmp_DVMag = DVMag(i) + tmp_DVMag;
end

avg_sma   = tmp_sma   / numPoints;
avg_ecc   = tmp_ecc   / numPoints;
avg_inc   = tmp_inc   / numPoints;
avg_raan  = tmp_raan  / numPoints;
avg_w     = tmp_w     / numPoints;
avg_TA    = tmp_TA    / numPoints;
avg_alat  = tmp_alat  / numPoints;
avg_DVMag = tmp_DVMag / numPoints;

if avg_alat > 360.0 
	avg_alat = avg_alat - 360.0;
end

%--- Calculate standard deviations

tmp_sma   = 0;
tmp_ecc   = 0;
tmp_inc   = 0;
tmp_raan  = 0;
tmp_w     = 0;
tmp_TA    = 0;
tmp_alat  = 0;
tmp_DVMag = 0;

for i = 1:numPoints
   tmp_sma    = tmp_sma    + (sma(i)   - avg_sma  )^2;
   tmp_ecc    = tmp_ecc    + (ecc(i)   - avg_ecc  )^2;
   tmp_inc    = tmp_inc    + (inc(i)   - avg_inc  )^2;
   tmp_raan   = tmp_raan   + (raan(i)  - avg_raan )^2;
   tmp_w      = tmp_w      + (w(i)     - avg_w    )^2;
   tmp_TA     = tmp_TA     + (TA(i)    - avg_TA   )^2;
   tmp_alat   = tmp_alat   + (alat(i)  - avg_alat )^2;
   tmp_DVMag  = tmp_DVMag + (DVMag(i) - avg_DVMag)^2;
end

sig_sma   = sqrt(tmp_sma ) /(numPoints - 1);
sig_ecc   = sqrt(tmp_ecc ) /(numPoints - 1);
sig_inc   = sqrt(tmp_inc ) /(numPoints - 1);
sig_raan  = sqrt(tmp_raan )/(numPoints - 1);
sig_w     = sqrt(tmp_w )   /(numPoints - 1);
sig_TA    = sqrt(tmp_TA )  /(numPoints - 1);
sig_alat  = sqrt(tmp_alat )/(numPoints - 1);
sig_DVMag = sqrt(tmp_DVMag)/(numPoints - 1);

if sig_alat > 360.0 
	sig_alat = sig_alat - 360.0;
end

%--- Print results

disp(' ');

printtemp = [ 'Results using ' num2str(numPoints) ' trial runs.'];
disp(printtemp);

printtemp = ['Value     Nominal      Average      Std Deviation '];
disp(printtemp);

printtemp = ['sma:      ' num2str(nom_sma) '    ' num2str(avg_sma) '      ' num2str(sig_sma) '     km'];
disp(printtemp);
printtemp = ['ecc:      ' num2str(nom_ecc) '    ' num2str(avg_ecc) '      ' num2str(sig_ecc)  '       '];
disp(printtemp);
printtemp = ['inc:      ' num2str(nom_inc) '      ' num2str(avg_inc) '        ' num2str(sig_inc)  '       deg'];
disp(printtemp);
printtemp = ['raan:     ' num2str(nom_raan) '     ' num2str(avg_raan) '       ' num2str(sig_raan) '       deg'];
disp(printtemp);
printtemp = ['w:        ' num2str(nom_w) '     ' num2str(avg_w) '        ' num2str(sig_w) '      deg'];
disp(printtemp);
printtemp = ['TA        ' num2str(nom_TA) '      ' num2str(avg_TA) '      ' num2str(sig_TA) '     deg'];
disp(printtemp);
printtemp = ['alat      ' num2str(nom_alat) '     ' num2str(avg_alat) '       ' num2str(sig_alat) '       deg'];
disp(printtemp);  
printtemp = ['DVMag     ' num2str(nom_DVMag) '     ' num2str(avg_DVMag) '       ' num2str(sig_DVMag) '       km/sec'];
disp(printtemp);  


% Plot the histograms:

numBins = max(20, 0.04*numPoints);

subplot(2,3,1); 
hist(sma,numBins);
xlabel('SMA km');
ylabel('Count');
title('Launch Dispersions: Semi-Major Axis (km)','FontSize',8);
grid on;

subplot(2,3,2); 
hist(ecc,numBins);
xlabel('Eccentricity');
ylabel('Count');
title('Launch Dispersions: Eccentricity','FontSize',8);
grid on;

subplot(2,3,3); 
hist(inc,numBins);
xlabel('Inclination (deg)');
ylabel('Count');
title('Launch Dispersions: Inclination','FontSize',8);
grid on;

subplot(2,3,4); 
hist(raan,numBins);
xlabel('RAAN (deg)');
ylabel('Count');
title('Launch Dispersions: Right Ascension of Ascending Node','FontSize',8);
grid on;

subplot(2,3,5); 
hist(alat,numBins);
xlabel('Argument of Latitude (deg)');
ylabel('Count');
title('Launch Dispersions: Argument of Latitude','FontSize',8);
grid on;

subplot(2,3,6); 
hist(DVMag,numBins);
xlabel('Total Delta-V (km/sec)');
ylabel('Count');
title('Launch Dispersions: Total Delta-V','FontSize',8);
grid on;

%--- Close

stkClose(conid)