% Matlab Astrogator Demonstration for Launch Errors Scenario
% Mike Loucks, SEE Corp:  www.see.com
% April 6, 2005

% Open STK/Matlab TCP/IP Connection

conid = stkopen('localhost:5001');
rtn = stkConnect(conid,'ConControl','/ VerboseOn AckOn');
disp('Connected to STK');

start_time = ['15 Dec 2001 00:00:00.00'];
stop_time  = ['25 Jan 2002 00:00:00.00'];

sat = ['*/Satellite/Sat1'];

% Get Values from Gator

tempLatitude      = stkExec(conid,'ASTROGATOR_RM */Satellite/Sat1 GETVALUE MainSEQUENCE.SegmentList.WallopsLaunch.Burnout.Geodetic.Latitude');
tempLongitude     = stkExec(conid,'ASTROGATOR_RM */Satellite/Sat1 GETVALUE MainSEQUENCE.SegmentList.WallopsLaunch.Burnout.Geodetic.Longitude');
tempAltitude      = stkExec(conid,'ASTROGATOR_RM */Satellite/Sat1 GETVALUE MainSEQUENCE.SegmentList.WallopsLaunch.Burnout.Geodetic.Altitude');
tempFixedVelocity = stkExec(conid,'ASTROGATOR_RM */Satellite/Sat1 GETVALUE MainSEQUENCE.SegmentList.WallopsLaunch.Burnout.FixedVelocity');

% Strip values out of string
initLatitude      = StripGatorValue(tempLatitude);
initLongitude     = StripGatorValue(tempLongitude);
initAltitude      = StripGatorValue(tempAltitude);
initFixedVelocity = StripGatorValue(tempFixedVelocity);

% Set Values in Gator

Altitude = 325;
Latitude = 49;
Longitude = -51;
FixedVelocity = 7.43;

varib = ['ASTROGATOR */Satellite/Sat1 SETVALUE MainSEQUENCE.SegmentList.WallopsLaunch.Burnout.Geodetic.Altitude ' num2str(Altitude) ' km'];   
rtn =  stkExec(conid,varib);

varib = ['ASTROGATOR */Satellite/Sat1 SETVALUE MainSEQUENCE.SegmentList.WallopsLaunch.Burnout.Geodetic.Latitude ' num2str(Latitude) ' deg'];   
rtn =  stkExec(conid,varib);

varib = ['ASTROGATOR */Satellite/Sat1 SETVALUE MainSEQUENCE.SegmentList.WallopsLaunch.Burnout.Geodetic.Longitude ' num2str(Longitude) ' deg'];   
rtn =  stkExec(conid,varib);
   
varib = ['ASTROGATOR */Satellite/Sat1 SETVALUE MainSEQUENCE.SegmentList.WallopsLaunch.Burnout.FixedVelocity  ' num2str(FixedVelocity) ' km/sec'];   
rtn =  stkExec(conid,varib);  

% --- Run the MCS
q = '"';
  
times = [q start_time q  ' ' q stop_time q ];
   
rtn = stkConnect(conid,'Propagate',sat,times);